/*
 *Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms and conditions in the IBM International Program License Agreement.
 *
 * Copyright IBM Corporation 2003, 2004. All Rights Reserved. 
 */
package superscript;

import com.rational.test.ft.script.*;
import com.rational.test.ft.vp.*;
import java.io.*;

/**
 * Add file based verification point methods to the script framework.
 */
public class FileSupportScript extends RationalTestScript
{
	
	/**
	 * Pull the contents from the supplied file and return the contents
	 * in <code>String</String> form.  An appropriate exception is thrown if the 
	 * file can not be found.
	 * 
	 * @param file The file to extract the contents from and convert to String form.
	 * @return The contents of the supplied file.
	 */
	protected byte[] getRawFileContents(File file)
	{
		byte[] result = null;
		if ( file != null && file.exists() )
		{
			FileInputStream input = null;
			try
			{
				// Read in the specified file ...
				input = new FileInputStream(file);
				result =  new byte[(int)file.length()];
				int bytesRead = 0;
				int offset = 0;
				final int length = result.length;
				while ( bytesRead != -1 && offset < length )
				{
					bytesRead = input.read(result, offset, length-offset);
					if ( bytesRead >= 0 )
						// bytesRead == -1 when end-of-file is reached
						offset += bytesRead;
				}
				input.close();
			}
			catch ( IOException ioexc )
			{
				throw new FileSupportException(ioexc.getClass().getName()+": "+ioexc.getMessage());
			}
		}
		else
			throw new FileSupportException("Invalid file specification: "+file);
		return result;
	}
	
	/**
	 * Pull the contents from the supplied file and return the contents
	 * in <code>String</String> form.  An appropriate exception is thrown if the 
	 * file can not be found.
	 * 
	 * @param file The file to extract the contents from and convert to String form.
	 * @return The contents of the supplied file.
	 */
	protected String getFileContents(File file)
	{
		byte[] result = getRawFileContents(file);
		return ( result != null ? new String(result) : null );
	}
	
	/**
	 * Test the contents of a supplied file against the contents of
	 * a previously captured baseline.  If this is the first time the 
	 * script has been run with this test the baseline will be created.
	 * The supplied file name must either be the complete file name
	 * of the file to be tested or the datastore relative name of the 
	 * file.  If the file can not be located in either way an appropriate
	 * exception will be thrown.
	 * 
	 * @param vpName The script relative unique name for the associated
	 *               verification point.
	 * @param fileName The complete or datastore relative name of the 
	 *                 file whose contents are to be compared.
	 * @return	<code>true</code> if the compares to the baseline succesfully.
	 */
	protected boolean fileContentsTest(String vpName, String fileName)
	{
		return fileContentsTest(vpName, getFile(fileName));
	}
	
	private File getFile(String fileName)
	{
		File file = new File(fileName);
		if ( !file.exists() )
		{
			file = new File((String)getOption(IOptionName.DATASTORE), fileName);
			if ( !file.exists() )
				throw new FileSupportException("Invalid file name specified to fileContentsTest: "+fileName);
		}
		return file;
	}
	
	/**
	 * Test the contents of a supplied file against the contents of
	 * a previously captured baseline.  If this is the first time the 
	 * script has been run with this test the baseline will be created.
	 * If the file does not exist an appropriate exception is thrown.
	 * 
	 * @param vpName The script relative unique name for the associated
	 *               verification point.
	 * @param file The file whose contents are to be compared.
	 * @return	<code>true</code> if the compares to the baseline succesfully.
	 */
	protected boolean fileContentsTest(String vpName, File file)
	{
		ITestData data = VpUtil.getTestData(getFileContents(file));
		return vpManual(vpName, data).performTest();
	}
	
	/**
	 * Test the contents of a supplied files against ecah other.
	 * The supplied file names must either be the complete file name
	 * of the file to be tested or the datastore relative name of the 
	 * file.  If either file can not be located an appropriate
	 * exception will be thrown.
	 * 
	 * @param vpName The script relative unique name for the associated
	 *               verification point.
	 * @param expected The complete or datastore relative name of the 
	 *                 expected file whose contents are to be compared.
	 * @param actual The complete or datastore relative name of the 
	 *                 actual file whose contents are to be compared.
	 * @return	<code>true</code> if the comparison of the supplied files is succesfully.
	 */
	protected boolean fileContentsTest(String vpName, String expected, String actual)
	{
		return fileContentsTest(vpName, getFile(expected), getFile(actual));
	}
	
	/**
	 * Test the contents of a supplied files against ecah other.
	 * The supplied file names must either be the complete file name
	 * of the file to be tested or the datastore relative name of the 
	 * file.  If either file can not be located an appropriate
	 * exception will be thrown.
	 * 
	 * @param vpName The script relative unique name for the associated
	 *               verification point.
	 * @param expected The complete or datastore relative name of the 
	 *                 expected file whose contents are to be compared.
	 * @param actual The complete or datastore relative name of the 
	 *                 actual file whose contents are to be compared.
	 * @return	<code>true</code> if the comparison of the supplied files is succesfully.
	 */
	protected boolean fileContentsTest(String vpName, File expected, File actual)
	{
		ITestData dataExp = VpUtil.getTestData(getFileContents(expected));
		ITestData dataAct = VpUtil.getTestData(getFileContents(actual));
		return vpManual(vpName, dataExp, dataAct).performTest();
	}
	
	public static class FileSupportException extends RuntimeException
	{
		public FileSupportException() 
		{
			super();
		}
		public FileSupportException(String message) 
		{
			super(message);
		}
	}
	
}
